/***********************************************************************
 * $Id: FXPy.cpp,v 1.15 2000/11/10 22:44:38 ljohnson Exp $
 ***********************************************************************/

#include "fx.h"
#include "FXJPEGImage.h"
#include "FXJPEGIcon.h"
#include "FXPNGIcon.h"
#include "FXPNGImage.h"
#include "fx3d.h"
#include "Python.h"
#include "FXPyApp.h"
#include "FXPy.h"
#include "impl.h"


extern "C" char* SWIG_GetPtr(char*,void**,char*);
extern "C" char* SWIG_GetPtrObj(PyObject*, void**, char*);
extern "C" void SWIG_MakePtr(char*,void*,char*);

extern FXAPI unsigned int fxTraceLevel;

// Maps FOX objects to Python objects
static FXDict FX2PyMap;
static PyObject *pPyDict = 0;

#ifdef FXPy_WITH_THREADS
extern "C" {
	PyThreadState*	g_FXPyEventThreadState = NULL;
}
#endif

static unsigned int	g_FXPyNestCount = 0;

static PyThreadState* myPyThreadState_Get() {
	PyThreadState* current = PyThreadState_Swap(NULL);
	PyThreadState_Swap(current);
	return current;
}

FXbool FXPyRestoreThread() {
#ifdef FXPy_WITH_THREADS
	g_FXPyNestCount++;
	if (g_FXPyEventThreadState != myPyThreadState_Get()) {
		PyEval_RestoreThread(g_FXPyEventThreadState);
		return TRUE;
	} else
#endif
	return FALSE;
}

void FXPySaveThread(FXbool doSave) {
#ifdef FXPy_WITH_THREADS
	if (doSave) {
		g_FXPyEventThreadState = PyEval_SaveThread();
	}
	g_FXPyNestCount--;
#endif
}

/*
*  Call this function to stop the FOX event loop and stop
*  the program (because an exception of some kind was raised).
*  If "msg" is NULL, assume that Python has already set the
*  error object and data and just quit. If "msg" is not NULL,
*  this indicates some kind of FXPy runtime error (i.e. a
*  bug) and we need to set the error object & string for that.
*/
void raiseException(const char *msg = 0) {
	
	if (msg != 0) {
		fxTraceLevel = 11;
		FXTRACE((10, "raiseException() with msg = \"%s\"\n", msg));
		PyErr_SetString(FXPyApp::errorObject, msg);
	}
	else
		FXTRACE((10, "raiseException(NULL)\n"));
	FXPyApp::pythonExceptionRaised=TRUE;
	FXPyApp::instance()->stop(EXIT_FAILURE);
}


// Store pointer to Python dictionary
PyObject* FXPySetDict(PyObject *, PyObject *args)
{
	FXbool doSave = FXPyRestoreThread();
	if (!PyArg_ParseTuple(args, "O", &pPyDict)) {
		FXPySaveThread(doSave);
		return NULL;
	}
	
	if (!PyDict_Check(pPyDict)) {
		PyErr_SetString(PyExc_TypeError,
			"FXPySetDict must have dictionary object!");
		FXPySaveThread(doSave);
		return NULL;
	}
	Py_INCREF(Py_None);
	FXPySaveThread(doSave);
	return Py_None;
}


// Register
void FXPyRegister(PyObject *pPyObject)
{
	FXObject *pFXObject = 0;
	SWIG_GetPtrObj(pPyObject, (void**) &pFXObject, "_FXObject_p");
	FX2PyMap.insert(FXStringFormat("%p", pFXObject).text(), pPyObject);
	FXbool doSave = FXPyRestoreThread();
	Py_INCREF(pPyObject);
	FXPySaveThread(doSave);
}


// Unregister
void FXPyUnregister(FXObject *pFXObject)
{
	PyObject *pPyObject = FXPyGetPyObject(pFXObject);
	FX2PyMap.remove(FXStringFormat("%p", pFXObject).text());
	FXbool doSave = FXPyRestoreThread();
	Py_DECREF(pPyObject); // dictionary's reference
	Py_DECREF(pPyObject); // other reference
	FXPySaveThread(doSave);
}


// Given a Python class instance which is a shadow for some
// real C++ object, return the corresponding FXObject pointer.
FXObject *FXPyGetFXObject(PyObject *pPyObject)
{
	FXObject *obj = 0;
 	SWIG_GetPtrObj(pPyObject, (void**) &obj, "_FXObject_p");
 	FXASSERT(obj != 0);
 	return obj;
}


/*
*  If the input object has a Python shadow class instance
*  counterpart, return a new reference to that instance.
*  Otherwise return NULL.
*/
PyObject *FXPyGetPyObject(const FXObject *pFXObject)
{
	FXString s = FXStringFormat("%p", pFXObject);
	PyObject *obj = (PyObject*) FX2PyMap.find(s.text());
	if (obj != 0) {
		FXbool doSave = FXPyRestoreThread();
		Py_INCREF(obj);
		FXPySaveThread(doSave);
		return obj;
	}
	else
		return 0;
}


// Given a class name like "FXList", convert it to "_FXPyList_p".
FXString FXPyGetSWIGPtrName(const char *s)
{
	FXString classname(s);
	if (classname.left(4) != FXString("FXPy"))
		classname = FXString("FXPy") + classname.right(classname.length()-2);
	return FXStringFormat("_%s_p", classname.text());
}


// Given an FXObject, return the FXPy Python class name
FXString FXPyGetPythonClassName(const FXObject *obj)
{
	FXString pythonClassName = obj->getClassName();
	if (pythonClassName.left(4) == "FXPy")
		pythonClassName = FXString("FX") + pythonClassName.right(pythonClassName.length() - 4);
	return pythonClassName;
}


/**
 * Return new reference to a Python shadow class instance for
 * this FOX object. If we recognize the input FOX object as one
 * that's already paired with a Python shadow class instance, we
 * return a new reference to that already-existing shadow class
 * instance. Otherwise, we need to construct a new instance of
 * that shadow class on-the-fly.
 *
 * Note that this is highly dependent on SWIG1.1's shadow class
 * implementation. If the passed-in pointer is NULL, or if some
 * error occurs, the error flag is set and the function returns
 * NULL.
 */
PyObject *
FXPyMakeShadowObject(const FXObject* obj)
{
	if (!obj)
		return NULL;

	// First see if it's one we already know about
	PyObject *shadow = FXPyGetPyObject(obj);
	if (shadow)
		return shadow;
	
	/**
	 *  A Python shadow class instance doesn't exist yet for this
	 *  C++ class instance, so we're going to have to instantiate
	 *  one on the fly.
	 */

	// First, determine the SWIG-mangled name for this class
	char buff[128];
	strcpy(buff, FXPyGetSWIGPtrName(obj->getClassName()).text());

	// Next, get the SWIG pointer representation (a string)
	char swigptr[128];
	SWIG_MakePtr(swigptr, (void*) obj, buff);
		
	// Get a pointer to the Python class object
	FXString pythonClassName = FXPyGetPythonClassName(obj) + "Ptr";
	strcpy(buff, pythonClassName.text());
	FXbool doSave = FXPyRestoreThread();
	PyObject *classobj = PyDict_GetItemString(pPyDict, buff);
	if (!classobj) {
		FXPySaveThread(doSave);
		return NULL;
	}

	// Construct input argument for constructor
	PyObject *args = Py_BuildValue("(s)", swigptr);
	if (!args) {
		FXPySaveThread(doSave);
		return NULL;
	}

	shadow = PyInstance_New(classobj, args, NULL);
	Py_DECREF(args);
	FXPySaveThread(doSave);
	return shadow;
}


// Return the class object for Python class FXEvent
static PyObject *
FXPyGetEventClassObject()
{
	static PyObject *EventClassObject = 0;
	if (EventClassObject == 0) {
		FXbool doSave = FXPyRestoreThread();
		EventClassObject = PyDict_GetItemString(pPyDict, "FXEventPtr");
		FXPySaveThread(doSave);
	}
	return EventClassObject;
}


// Return the class object for Python class FXTreeItem
static PyObject *
FXPyGetTreeItemClassObject()
{
	static PyObject *classObject = 0;
	if (classObject == 0) {
		FXbool doSave = FXPyRestoreThread();
		classObject = PyDict_GetItemString(pPyDict, "FXTreeItemPtr");
		FXPySaveThread(doSave);
	}
	return classObject;
}


// Returns message data appropriate for a given sender/type combination
static PyObject *
FXPyGetMessageData(FXObject *sender, FXSelector sel, void *ptr)
{
	FXASSERT(sender != 0);
	FXushort type = SELTYPE(sel);
	if (type == SEL_PAINT ||
		type == SEL_LEFTBUTTONPRESS ||
		type == SEL_LEFTBUTTONRELEASE ||
		type == SEL_MOTION) {
		// Data is pointer to the FXEvent record
		char swigptr[128];
		SWIG_MakePtr(swigptr, ptr, "_FXEvent_p");
		PyObject *eventClass = FXPyGetEventClassObject();
		FXbool doSave = FXPyRestoreThread();
		PyObject *args = Py_BuildValue("(s)", swigptr);
		PyObject *data = PyInstance_New(eventClass, args, NULL);
		Py_DECREF(args);
		FXPySaveThread(doSave);
		return data;
	}
	else if (sender->isMemberOf(FXMETACLASS(FXFileList))) {
		FXint active = (FXint) ptr;
		if (type == SEL_CHANGED || type == SEL_CLICKED ||
		    type == SEL_DOUBLECLICKED || type == SEL_TRIPLECLICKED ||
		    type == SEL_COMMAND) {
			FXbool doSave = FXPyRestoreThread();
			PyObject *result = Py_BuildValue("i", active);
			FXPySaveThread(doSave);
			return result;
		}
	}
	else if (sender->isMemberOf(FXMETACLASS(FXHeader))) {
		FXint active = (FXint) ptr;
		if (type == SEL_COMMAND || type == SEL_CHANGED) {
			FXbool doSave = FXPyRestoreThread();
			PyObject *result = Py_BuildValue("i", active);
			FXPySaveThread(doSave);
			return result;
		}
	}
	else if (sender->isMemberOf(FXMETACLASS(FXList))) {
		FXint active = (FXint) ptr;
		if (type == SEL_COMMAND || type == SEL_CHANGED) {
			FXbool doSave = FXPyRestoreThread();
			PyObject *result = Py_BuildValue("i", active);
			FXPySaveThread(doSave);
			return result;
		}
	}
	else if (sender->isMemberOf(FXMETACLASS(FXRecentFiles))) {
		FXchar *filename = (FXchar*) ptr;
		if (type == SEL_COMMAND) {
			FXbool doSave = FXPyRestoreThread();
			PyObject *result = Py_BuildValue("s", filename);
			FXPySaveThread(doSave);
			return result;
		}
	}
	else if (sender->isMemberOf(FXMETACLASS(FXTreeList))) {
		// FXTreeItem *item = (FXTreeItem*) ptr;
		if (type == SEL_COMMAND || type == SEL_CHANGED) {
			PyObject *data = FXPyGetPyObject((const FXObject*) ptr);
			if (data)
				return data;
			char swigptr[128];
			SWIG_MakePtr(swigptr, ptr, "_FXPyTreeItem_p");
			PyObject *eventClass = FXPyGetTreeItemClassObject();
			FXbool doSave = FXPyRestoreThread();
			PyObject *args = Py_BuildValue("(s)", swigptr);
			data = PyInstance_New(eventClass, args, NULL);
			Py_DECREF(args);
			FXPySaveThread(doSave);
			return data;
		}
	}

	if (ptr == 0) {
		FXbool doSave = FXPyRestoreThread();
		Py_INCREF(Py_None);
		FXPySaveThread(doSave);
		return Py_None;
	}
	else {
		char s[256];
		sprintf(s, "%p", ptr);
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyString_FromString(s);
		FXPySaveThread(doSave);
		return result;
	}
}


// Check for handle
FXbool FXPyCanHandle(FXObject* self, FXSelector sel)
{
	FXbool result = FALSE;
	PyObject* pPyObject = FXPyGetPyObject(self);
	if (pPyObject) {
		FXbool doSave = FXPyRestoreThread();
		if (PyObject_HasAttrString(pPyObject, "_search")) {
			PyObject *pMethObj = PyObject_CallMethod(pPyObject, "_search",
				"(i)", sel);
			Py_DECREF(pPyObject);
			if (pMethObj) {
				if (pMethObj == Py_None) {
					Py_DECREF(pMethObj);
					result = FALSE;
				}
				else {
					Py_DECREF(pMethObj);
					result = TRUE;
				}
			}
		}
		else {
			Py_DECREF(pPyObject);
			raiseException("object didn't define an _search() method");
		}
		FXPySaveThread(doSave);
	}
	return result;
}


// Handle message
long FXPyHandle(FXObject* self, FXObject* sender, FXSelector sel, void* ptr)
{
	// Get Python object for this C++ object
	PyObject *pPyObject = FXPyGetPyObject(self);
	if (pPyObject) {
		// Switch contexts
		FXbool doSave = FXPyRestoreThread();

		// Call _search() to get the method associated with this selector
		PyObject *pMethObj = PyObject_CallMethod(pPyObject, "_search", "(i)", sel);
		if (!pMethObj) {
			Py_DECREF(pPyObject);
			raiseException();
			FXPySaveThread(doSave);
			return 0;
		}
		
		// If _search() returns None then we don't want to handle this one
		if (pMethObj == Py_None) {
			Py_DECREF(Py_None);
			Py_DECREF(pPyObject);
			FXPySaveThread(doSave);
			return 0;
		}
		
		// Make sure it's really a callable object
		if (!PyCallable_Check(pMethObj)) {
			PyErr_SetString(PyExc_TypeError, "not a callable object");
			Py_DECREF(pMethObj);
			Py_DECREF(pPyObject);
			raiseException();
			FXPySaveThread(doSave);
			return 0;
		}
		
		// The sender may or may not have a Python counterpart, depending
		// on whether it was created in Python code or its some kind of
		// object created by FOX itself.
		PyObject *pSender = FXPyGetPyObject(sender);
		if (!pSender)
			pSender = FXPyMakeShadowObject(sender);

		if (pSender) {
			// Convert message data to something usable in Python
			PyObject *data = FXPyGetMessageData(sender, sel, ptr);

			// Call the message handler function (an unbound method)
			PyObject *pResObj = PyObject_CallFunction(pMethObj, "OOiO",
				pPyObject, pSender, sel, data);

			Py_DECREF(data);
			Py_DECREF(pMethObj);
			Py_DECREF(pPyObject);
			
			// Message handler must return an integer value
			if (pResObj) {
				if (PyInt_Check(pResObj)) {
					long nRes = PyInt_AsLong(pResObj);
					Py_DECREF(pResObj);
					FXPySaveThread(doSave);
					return nRes;
				}
				else {
					Py_DECREF(pResObj);
					raiseException("message handler function must return an integer");
					FXPySaveThread(doSave);
					return 1;
				}
			}
			else {
				raiseException();
				FXPySaveThread(doSave);
				return 1;
			}
		}
		else {
			Py_DECREF(pPyObject);
			Py_DECREF(pMethObj);
			raiseException();
			FXPySaveThread(doSave);
			return 0;
		}
	}
	return 1;
}


// Call a method named "func" of the shadow class "obj".
// The method is assumed to return None.
static void FXPyCallVoidFunction(FXObject *obj, char *func)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);

	if (pPyObject) {
		// Call the function
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func, NULL);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				FXPySaveThread(doSave);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
}


// Call a method named "func" of the shadow class "obj", passing
// it the shadow class instance for "arg1" as its single argument.
// The method is assumed to return None.
static void FXPyCallVoidFunction(FXObject *obj, char *func, PyObject *arg1)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);

	if (pPyObject) {
		// Call the function
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func,
			"(O)", arg1);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				FXPySaveThread(doSave);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func, FXObject* arg1)
{
	PyObject *pyarg1 = FXPyGetPyObject(arg1);
	FXPyCallVoidFunction(obj, func, pyarg1);
	FXbool doSave = FXPyRestoreThread();
	Py_DECREF(pyarg1);
	FXPySaveThread(doSave);
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func,
		     PyObject *arg1, PyObject *arg2)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);

	if (pPyObject) {
		// Call the function
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func, "(OO)",
			arg1, arg2);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				FXPySaveThread(doSave);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
}


static void
FXPyCallVoidFunction(FXObject *obj, char *func,
		     PyObject *arg1, PyObject *arg2,
		     PyObject *arg3, PyObject *arg4)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		// Call the function
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func,
			"(OOOO)", arg1, arg2, arg3, arg4);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				FXPySaveThread(doSave);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func, FXint i)
{
	FXbool doSave = FXPyRestoreThread();
	PyObject *arg1 = PyInt_FromLong(i);
	FXPyCallVoidFunction(obj, func, arg1);
	Py_DECREF(arg1);
	FXPySaveThread(doSave);
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func,
                     FXint i, FXint j)
{
	FXbool doSave = FXPyRestoreThread();
	PyObject *arg1 = PyInt_FromLong(i);
	PyObject *arg2 = PyInt_FromLong(j);
	FXPyCallVoidFunction(obj, func, arg1, arg2);
	Py_DECREF(arg2);
	Py_DECREF(arg1);
	FXPySaveThread(doSave);
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func,
                     FXint x, FXint y, FXint w, FXint h)
{
	FXbool doSave = FXPyRestoreThread();
	PyObject *arg1 = PyInt_FromLong(x);
	PyObject *arg2 = PyInt_FromLong(y);
	PyObject *arg3 = PyInt_FromLong(w);
	PyObject *arg4 = PyInt_FromLong(h);
	FXPyCallVoidFunction(obj, func, arg1, arg2, arg3, arg4);
	Py_DECREF(arg4);
	Py_DECREF(arg3);
	Py_DECREF(arg2);
	Py_DECREF(arg1);
	FXPySaveThread(doSave);
}


// Call a method named "func" of the shadow class "obj".
// The method is assumed to return a boolean value.
static FXbool FXPyCallBoolFunction(const FXObject* obj, char *func)
{
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func, NULL);
		Py_DECREF(pPyObject);
		if (result) {
			if (PyInt_Check(result)) {
				FXbool bRes = (FXbool) PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return bRes;
			}
			else {
				Py_DECREF(result);
				raiseException("Expected function to return 0 or 1");
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
	return FALSE;
}


// Call a method named "func" of the shadow class "obj".
// The method is assumed to return an FXint value.
static FXint FXPyCallIntFunction(FXObject* obj, char *func)
{
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func, NULL);
		Py_DECREF(pPyObject);
		if (result) {
			if (PyInt_Check(result)) {
				FXint nResult = (FXint) PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return nResult;
			}
			else {
				Py_DECREF(result);
				raiseException("Expected function to return an integer");
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
	return -1;
}

static FXint FXPyCallIntFunction(FXObject* obj, char *func, PyObject *arg1)
{
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pPyObject, func, "(O)",
			arg1);
		Py_DECREF(pPyObject);
		if (result) {
			if (PyInt_Check(result)) {
				FXint nResult = (FXint) PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return nResult;
			}
			else {
				Py_DECREF(result);
				raiseException("Expected function to return an integer");
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
	return -1;
}


void FXPyCreate(FXObject *obj) { FXPyCallVoidFunction(obj, "create"); }
void FXPyDetach(FXObject *obj) { FXPyCallVoidFunction(obj, "detach"); }
void FXPyDestroy(FXObject *obj) { FXPyCallVoidFunction(obj, "destroy"); }
void FXPyShow(FXObject *obj) { FXPyCallVoidFunction(obj, "show"); }


// long FXObject::onDefault(FXObject* sender,FXSelector sel,void* ptr)
static long
_onDefault(FXObject* fxObj,FXObject* sender,FXSelector sel,void* ptr) {
	PyObject *pyObj = FXPyGetPyObject(fxObj);
	if (pyObj != NULL) {
		PyObject *arg0 = FXPyMakeShadowObject(sender);
		if (arg0 == NULL) {
			FXbool doSave = FXPyRestoreThread();
			Py_DECREF(pyObj);
			FXPySaveThread(doSave);
			raiseException();
			return 0;
		}
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(pyObj, "onDefault", "(Oii)", arg0, sel, 0);
		Py_DECREF(arg0);
		Py_DECREF(pyObj);
		if (result != NULL) {
			if (PyInt_Check(result)) {
				long longResult = PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return longResult;
			}
			else {
				Py_DECREF(result);
				raiseException("onDefault() should return an integer");
			}
			Py_DECREF(result);
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
	return 0;
}


// FXWindow::getDefaultWidth() const
static FXint
_getDefaultWidth(const FXWindow* win) {
	PyObject *obj = FXPyGetPyObject(win);
	if (obj) {
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(obj, "getDefaultWidth", NULL);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint defaultWidth = PyInt_AsLong(result);
				FXASSERT(defaultWidth >= 0);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return defaultWidth;
			}
			else {
				Py_DECREF(result);
				raiseException("getDefaultWidth() should return an integer");
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
	return -1;
}


// FXWindow::getDefaultHeight() const
static FXint _getDefaultHeight(const FXWindow* win) {
	PyObject *obj = FXPyGetPyObject(win);
	if (obj) {
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(obj, "getDefaultHeight", NULL);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint defaultHeight = PyInt_AsLong(result);
				FXASSERT(defaultHeight >= 0);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return defaultHeight;
			}
			else {
				Py_DECREF(result);
				raiseException("getDefaultHeight() should return an integer");
			}
		}
		else {
			raiseException();
		}
		FXPySaveThread(doSave);
	}
	return -1;
}


// FXWindow::layout()
static void _layout(FXWindow* win) { FXPyCallVoidFunction(win, "layout"); }


template <class ItemType, class ListType>
FXint _getWidth(const ItemType *self, const ListType *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			FXbool doSave = FXPyRestoreThread();
			Py_DECREF(obj);
			FXPySaveThread(doSave);
			raiseException();
			return -1;
		}
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(obj, "getWidth", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint w = PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return w;
			}
			else {
				Py_DECREF(result);
				raiseException("getWidth() should return an integer");
				FXPySaveThread(doSave);
				return -1;
			}
		}
		else {
			raiseException();
			FXPySaveThread(doSave);
		}
	}
	return -1;
}

template <class ItemType, class ListType>
FXint _getHeight(const ItemType *self, const ListType *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			FXbool doSave = FXPyRestoreThread();
			Py_DECREF(obj);
			FXPySaveThread(doSave);
			raiseException();
			return -1;
		}
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(obj,
			"getHeight", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint h = PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return h;
			}
			else {
				Py_DECREF(result);
				raiseException("getHeight() should return an integer");
				FXPySaveThread(doSave);
				return -1;
			}
		}
		else {
			raiseException();
			FXPySaveThread(doSave);
		}
	}
	return -1;
}

// void FXGLObject::draw(FXGLViewer* viewer)
static void _draw(FXGLObject* self, FXGLViewer* viewer) {
	FXPyCallVoidFunction(self, "draw", viewer);
}


// void FXGLObject::hit(FXGLViewer* viewer)
static void _hit(FXGLObject* self, FXGLViewer* viewer) {
	FXPyCallVoidFunction(self, "hit", viewer);
}


// FXbool FXGLObject::canDrag()
static FXbool _canDrag(const FXGLObject* self) {
	return FXPyCallBoolFunction(self, "canDrag");
}


// FXbool FXGLObject::canDelete()
static FXbool _canDelete(const FXGLObject* self) {
	return FXPyCallBoolFunction(self, "canDelete");
}


// FXbool FXGLObject::drag(FXGLViewer* viewer, FXint fx, FXint fy, FXint tx, FXint ty)
static FXbool
_drag(FXGLObject* self, FXGLViewer* viewer,
      FXint fx, FXint fy, FXint tx, FXint ty) {
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *v = FXPyGetPyObject(viewer);
		if (!v) {
			FXbool doSave = FXPyRestoreThread();
			Py_DECREF(obj);
			FXPySaveThread(doSave);
			raiseException();
			return FALSE;
		}

		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(obj, "drag",
			"(Oiiii)", v, fx, fy, tx, ty);
		Py_DECREF(v);
		Py_DECREF(obj);

		if (result) {
			if (PyInt_Check(result)) {
				FXbool bRes = (FXbool) PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return bRes;
			}
			else {
				Py_DECREF(result);
				raiseException("drag() should return either zero or one");
				FXPySaveThread(doSave);
				return FALSE;
			}
		}
		else {
			raiseException();
			FXPySaveThread(doSave);
			return FALSE;
		}
	}
	return FALSE;
}



static void _setText(FXObject *self, const FXString& text)
{
	FXbool doSave = FXPyRestoreThread();
	PyObject *s = PyString_FromString(text.text());
	if (s) {
		FXPyCallVoidFunction(self, "setText", s);
		Py_DECREF(s);
	}
	else
		raiseException();
	FXPySaveThread(doSave);
}


static void _setIcon(FXObject *self, FXIcon *icn) {
	FXPyCallVoidFunction(self, "setIcon", icn);
}


static void _setMiniIcon(FXIconItem *self, FXIcon *icn) {
	FXPyCallVoidFunction(self, "setMiniIcon", icn);
}


static void _setBigIcon(FXIconItem *self, FXIcon *icn) {
	FXPyCallVoidFunction(self, "setBigIcon", icn);
}


static void _setOpenIcon(FXTreeItem *self, FXIcon *icn) {
	FXPyCallVoidFunction(self, "setOpenIcon", icn);
}


static void _setClosedIcon(FXTreeItem *self, FXIcon *icn) {
	FXPyCallVoidFunction(self, "setClosedIcon", icn);
}


static void _setFocus(FXObject *self, FXbool focus) {
	FXbool doSave = FXPyRestoreThread();
	PyObject *i = PyInt_FromLong((long) focus);
	if (i) {
		FXPyCallVoidFunction(self, "setFocus", i);
		Py_DECREF(i);
	}
	else
		raiseException();
	FXPySaveThread(doSave);
}

static void _setSelected(FXObject *self, FXbool selected) {
	FXbool doSave = FXPyRestoreThread();
	PyObject *i = PyInt_FromLong((long) selected);
	if (i) {
		FXPyCallVoidFunction(self, "setSelected", i);
		Py_DECREF(i);
	}
	else
		raiseException();
	FXPySaveThread(doSave);
}

static void _setOpened(FXTreeItem *self, FXbool opened) {
    FXbool doSave = FXPyRestoreThread();
    PyObject *i = PyInt_FromLong((long) opened);
    if (i) {
	FXPyCallVoidFunction(self, "setOpened", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
    FXPySaveThread(doSave);
}

static void _setExpanded(FXTreeItem *self, FXbool expanded) {
    FXbool doSave = FXPyRestoreThread();
    PyObject *i = PyInt_FromLong((long) expanded);
    if (i) {
	FXPyCallVoidFunction(self, "setExpanded", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
    FXPySaveThread(doSave);
}

static void _setEnabled(FXObject *self, FXbool enabled) {
    FXbool doSave = FXPyRestoreThread();
    PyObject *i = PyInt_FromLong((long) enabled);
    if (i) {
	FXPyCallVoidFunction(self, "setEnabled", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
    FXPySaveThread(doSave);
}

static void _setDraggable(FXObject *self, FXbool draggable) {
    FXbool doSave = FXPyRestoreThread();
    PyObject *i = PyInt_FromLong((long) draggable);
    if (i) {
	FXPyCallVoidFunction(self, "setDraggable", i);
	Py_DECREF(i);
    }
    else
	raiseException();
    FXPySaveThread(doSave);
}

static void _setMarked(FXObject *self, FXbool marked) {
    FXbool doSave = FXPyRestoreThread();
    PyObject *i = PyInt_FromLong((long) marked);
    if (i) {
	FXPyCallVoidFunction(self, "setMarked", i);
	Py_DECREF(i);
    }
    else
	raiseException();
    FXPySaveThread(doSave);
}

static void _setJustify(FXTableItem *self, FXuint justify) {
    FXbool doSave = FXPyRestoreThread();
    PyObject *i = PyInt_FromLong((long) justify);
    if (i) {
	FXPyCallVoidFunction(self, "setJustify", i);
	Py_DECREF(i);
    }
    else
	raiseException();
    FXPySaveThread(doSave);
}


// void FXWindow::enable()
static void _enable(FXWindow *self) {
	FXPyCallVoidFunction(self, "enable");
}


// void FXWindow::disable()
static void _disable(FXWindow *self) {
	FXPyCallVoidFunction(self, "disable");
}


// void FXWindow::hide()
static void _hide(FXWindow *self) {
	FXPyCallVoidFunction(self, "hide");
}


// void FXWindow::lower()
static void _lowerWindow(FXWindow *self) {
	FXPyCallVoidFunction(self, "lower");
}


// FXint FXWindow::getWidthForHeight(FXint h)
static FXint _getWidthForHeight(FXWindow *self, FXint h) {
    FXint w = -1;
    FXbool doSave = FXPyRestoreThread();
    PyObject *o = PyInt_FromLong((long) h);
    if (o) {
	w = FXPyCallIntFunction(self, "getWidthForHeight", o);
	Py_DECREF(o);
    }
    else {
	raiseException();
    }
    FXPySaveThread(doSave);
    return w;
}


// FXint FXWindow::getHeightForWidth(FXint w)
static FXint _getHeightForWidth(FXWindow *self, FXint w) {
    FXint h = -1;
    FXbool doSave = FXPyRestoreThread();
    PyObject *o = PyInt_FromLong((long) w);
    if (o) {
	h = FXPyCallIntFunction(self, "getHeightForWidth", o);
	Py_DECREF(o);
    }
    else {
	raiseException();
    }
    FXPySaveThread(doSave);
    return h;
}


// void FXWindow::recalc()
static void _recalc(FXWindow *self) {
	FXPyCallVoidFunction(self, "recalc");
}


// FXbool FXWindow::isComposite() const
static FXbool _isComposite(const FXWindow *self) {
	return FXPyCallBoolFunction(self, "isComposite");
}


// void FXWindow::move(FXint x, FXint y)
static void _move(FXWindow *self, FXint x, FXint y) {
	FXbool doSave = FXPyRestoreThread();
	PyObject *arg1 = PyInt_FromLong(x);
	PyObject *arg2 = PyInt_FromLong(y);
	if (arg1 && arg2) {
		FXPyCallVoidFunction(self, "move", arg1, arg2);
		Py_DECREF(arg2);
		Py_DECREF(arg1);
	}
	else {
		Py_XDECREF(arg1);
		Py_XDECREF(arg2);
		raiseException();
	}
	FXPySaveThread(doSave);
}


// void FXDrawable::resize(FXint w, FXint h)
static void _resize(FXDrawable *self, FXint w, FXint h) {
	FXbool doSave = FXPyRestoreThread();
	PyObject *arg1 = PyInt_FromLong(w);
	PyObject *arg2 = PyInt_FromLong(h);
	if (arg1 && arg2) {
		FXPyCallVoidFunction(self, "resize", arg1, arg2);
		Py_DECREF(arg2);
		Py_DECREF(arg1);
	}
	else {
		Py_XDECREF(arg1);
		Py_XDECREF(arg2);
		raiseException();
	}
	FXPySaveThread(doSave);
}


// void FXWindow::position(FXint x, FXint y, FXint w, FXint h)
static void _position(FXWindow *self, FXint x, FXint y, FXint w, FXint h) {
	FXbool doSave = FXPyRestoreThread();
	PyObject *arg1 = PyInt_FromLong(x);
	PyObject *arg2 = PyInt_FromLong(y);
	PyObject *arg3 = PyInt_FromLong(w);
	PyObject *arg4 = PyInt_FromLong(h);
	if (arg1 && arg2 && arg3 && arg4) {
		FXPyCallVoidFunction(self, "position", arg1, arg2, arg3, arg4);
		Py_DECREF(arg4);
		Py_DECREF(arg3);
		Py_DECREF(arg2);
		Py_DECREF(arg1);
	}
	else {
		Py_XDECREF(arg1);
		Py_XDECREF(arg2);
		Py_XDECREF(arg3);
		Py_XDECREF(arg4);
	}
	FXPySaveThread(doSave);
}


// FXbool FXWindow::canFocus()
static FXbool _canFocus(const FXWindow *self) {
	return FXPyCallBoolFunction(self, "canFocus");
}


// void FXWindow::setFocus()
static void _setFocus(FXWindow *self) {
	FXPyCallVoidFunction(self, "setFocus");
}


// void FXWindow::killFocus()
static void _killFocus(FXWindow *self) {
	FXPyCallVoidFunction(self, "killFocus");
}


// FXbool FXWindow::contains(FXint x, FXint y)
static FXbool _contains(const FXWindow *self, FXint x, FXint y) {
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		FXbool doSave = FXPyRestoreThread();
		PyObject *result = PyObject_CallMethod(obj, "contains",
			"(ii)", x, y);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXbool bRes = (FXbool) PyInt_AsLong(result);
				Py_DECREF(result);
				FXPySaveThread(doSave);
				return bRes;
			}
			else {
				Py_DECREF(result);
				raiseException("contains() should return either zero or one");
				FXPySaveThread(doSave);
				return FALSE;
			}
		}
		else {
			raiseException();
			FXPySaveThread(doSave);
			return FALSE;
		}
	}
	else {
		raiseException();
		return FALSE;
	}
}

// FXbool FXWindow::doesSaveUnder() const
static FXbool _doesSaveUnder(const FXWindow *self) {
	return FXPyCallBoolFunction(self, "doesSaveUnder");
}

static void _restore(FXImage *self) {
	FXPyCallVoidFunction(self, "restore");
}

static void _render(FXImage *self) {
	FXPyCallVoidFunction(self, "render");
}

static void _scale(FXImage *self, FXint w, FXint h) {
	FXPyCallVoidFunction(self, "scale", w, h);
}

static void _mirror(FXImage *self, FXbool horizontal, FXbool vertical) {
	FXPyCallVoidFunction(self, "mirror", horizontal, vertical);
}

static void _rotate(FXImage *self, FXint degrees) {
	FXPyCallVoidFunction(self, "rotate", degrees);
}

static void _crop(FXImage *self, FXint x, FXint y, FXint w, FXint h) {
	FXPyCallVoidFunction(self, "crop", x, y, w, h);
}

static void _savePixels(const FXImage *self, FXStream& store) {
	self->savePixels(store);
}

static void _loadPixels(FXImage *self, FXStream& store) {
	self->loadPixels(store);
}

static FXint _getContentWidth(FXScrollArea *self) {
	return FXPyCallIntFunction(self, "getContentWidth");
}

static FXint _getContentHeight(FXScrollArea *self) {
	return FXPyCallIntFunction(self, "getContentHeight");
}

static FXint _getViewportWidth(FXScrollArea *self) {
	return FXPyCallIntFunction(self, "getViewportWidth");
}

static FXint _getViewportHeight(FXScrollArea *self) {
	return FXPyCallIntFunction(self, "getViewportHeight");
}

static void _moveContents(FXScrollArea *self, FXint x, FXint y) {
	FXPyCallVoidFunction(self, "moveContents", x, y);
}

static void _minimize(FXMDIChild *self, FXbool notify) {
	FXPyCallVoidFunction(self, "minimize", notify);
}

static void _maximize(FXMDIChild *self, FXbool notify) {
	FXPyCallVoidFunction(self, "maximize", notify);
}

static void _restore(FXMDIChild *self, FXbool notify) {
	FXPyCallVoidFunction(self, "restore", notify);
}

static void _dock(FXToolbar *self) {
	FXPyCallVoidFunction(self, "dock");
}

static void _undock(FXToolbar *self) {
	FXPyCallVoidFunction(self, "undock");
}

#include "virtuals.cpp"
