#! /usr/bin/env python

from FXPy import *

try:
    from OpenGL.GL import *
    from OpenGL.GLU import *
except:
    print "Sorry, couldn't import the PyOpenGL module."
    raise SystemExit


# Subclass of tab book
class TabBook(FXTabBook):
    def __init__(self, frame, parent):
        FXTabBook.__init__(self, frame)
        self.makeAnglesTab(parent)
        self.makeColorsTab(parent)
        self.makeSwitchesTab(parent)

    def makeAnglesTab(self, parent):
        FXTabItem(self, "Angles\tCamera Angles\tSwitch to camera angles panel.")
        angles = FXMatrix(self,2,FRAME_THICK|FRAME_RAISED|MATRIX_BY_COLUMNS|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)

        FXLabel(angles,"X:")
        x_dial = FXDial(angles,parent.mdiclient,FXGLViewer.ID_DIAL_X,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,150,14,0,0,0,0)
        x_dial.setTipText("Rotate about X")

        FXLabel(angles,"Y:")
        y_dial = FXDial(angles,parent.mdiclient,FXGLViewer.ID_DIAL_Y,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,150,14,0,0,0,0)
        y_dial.setTipText("Rotate about Y")

        FXLabel(angles,"Z:")
        z_dial = FXDial(angles,parent.mdiclient,FXGLViewer.ID_DIAL_Z,FRAME_SUNKEN|FRAME_THICK|DIAL_CYCLIC|DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_CENTER_Y,0,0,150,14,0,0,0,0)
        z_dial.setTipText("Rotate about Z")

        FXLabel(angles,"FOV:")
        fov = FXTextField(angles,10,parent.mdiclient,FXGLViewer.ID_FOV,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK)
        fov.setTipText("Field of view")

        FXLabel(angles,"Zoom:")
        zz = FXTextField(angles,10,parent.mdiclient,FXGLViewer.ID_ZOOM,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK)
        zz.setTipText("Zooming")

    def makeColorsTab(self, parent):
        FXTabItem(self, "Colors\tColors\tSwitch to color panel.")
        colors = FXMatrix(self,2,MATRIX_BY_COLUMNS|FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_CENTER_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)
        FXLabel(colors,"Background:",None,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT)
        FXColorWell(colors,0,parent.mdiclient,FXGLViewer.ID_BACK_COLOR,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0)
        FXLabel(colors,"Ambient:",None,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT)
        FXColorWell(colors,0,parent.mdiclient,FXGLViewer.ID_AMBIENT_COLOR,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0)


        FXLabel(colors,"Light Amb:",None,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT)
        FXColorWell(colors,0,parent.mdiclient,FXGLViewer.ID_LIGHT_AMBIENT,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0)
        FXLabel(colors,"Light Diff:",None,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT)
        FXColorWell(colors,0,parent.mdiclient,FXGLViewer.ID_LIGHT_DIFFUSE,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0)
        FXLabel(colors,"Light Spec:",None,LAYOUT_RIGHT|LAYOUT_CENTER_Y|JUSTIFY_RIGHT)
        FXColorWell(colors,0,parent.mdiclient,FXGLViewer.ID_LIGHT_SPECULAR,LAYOUT_TOP|LAYOUT_LEFT, 0,0,0,0 ,0,0,0,0)

    def makeSwitchesTab(self, parent):
        # Switches
        FXTabItem(self,"Settings\tSettings\tSwitch to settings panel.")
        settings = FXVerticalFrame(self,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_CENTER_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)
        FXCheckButton(settings,"Lighting",parent.mdiclient,FXGLViewer.ID_LIGHTING,ICON_BEFORE_TEXT)
        FXCheckButton(settings,"Fog",parent.mdiclient,FXGLViewer.ID_FOG,ICON_BEFORE_TEXT)
        FXCheckButton(settings,"Dither",parent.mdiclient,FXGLViewer.ID_DITHER,ICON_BEFORE_TEXT)
        FXCheckButton(settings,"Lock",parent.mdiclient,FXGLViewer.ID_LOCK,ICON_BEFORE_TEXT)
        FXCheckButton(settings,"Turbo",parent.mdiclient,FXGLViewer.ID_TURBO,ICON_BEFORE_TEXT)

# Test for OpenGL
class GLViewWindow(FXMainWindow):
    ID_ABOUT = FXMainWindow.ID_LAST + 1
    ID_OPEN = ID_ABOUT + 1
    ID_NEWVIEWER = ID_OPEN + 1
    ID_QUERY_MODE = ID_NEWVIEWER + 1

    def iconResource(self, filename, transp=0, opts=0):
        return FXGIFIcon(self.getApp(), open(filename, 'rb').read(),
            transp, opts)

    # Constructor
    def __init__(self,app):
        # Must invoke base class constructor explicitly
        FXMainWindow.__init__(self,app,"OpenGL Test Application",w=800,h=600)

        # Message map
        FXMAPFUNC(self,SEL_COMMAND,GLViewWindow.ID_ABOUT,GLViewWindow.onCmdAbout)
        FXMAPFUNC(self,SEL_COMMAND,GLViewWindow.ID_OPEN,GLViewWindow.onCmdOpen)
        FXMAPFUNC(self,SEL_COMMAND,GLViewWindow.ID_NEWVIEWER,GLViewWindow.onCmdNewViewer)
        FXMAPFUNC(self,SEL_UPDATE,GLViewWindow.ID_QUERY_MODE,GLViewWindow.onUpdMode)

        peng = self.iconResource('gifs/penguin.gif')
        self.setIcon(peng)

        self.getAccelTable().addAccel(fxparseaccel("Ctl-Q"),self.getApp(),MKUINT(FXApp.ID_QUIT,SEL_COMMAND))
        colordlg = FXColorDialog(self,"Color Dialog")

        # Menubar
        menubar = FXMenubar(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)

        # Tool bar
        FXHorizontalSeparator(self,LAYOUT_SIDE_TOP|SEPARATOR_GROOVE|LAYOUT_FILL_X)
        toolbar = FXHorizontalFrame(self,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, 4,4,0,0, 0,0)

        # Make status bar
        statusbar = FXStatusbar(self, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER)

        # Senor Tux
        FXButton(statusbar, "\tHello, I'm Tux...\nThe symbol for the Linux Operating System.\nAnd all it stands for.",
            self.iconResource('gifs/penguin.gif'), opts = LAYOUT_RIGHT)

        # Contents
        frame = FXHorizontalFrame(self,
            LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0, 0,0)

        # Nice sunken box around the GL viewer
        box = FXVerticalFrame(frame,
            FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
            0,0,0,0, 0,0,0,0)

        # MDI client area
        self.mdiclient = FXMDIClient(box, LAYOUT_FILL_X|LAYOUT_FILL_Y)

        # MDI buttons in menu
        FXMDIWindowButton(menubar, self.mdiclient, FXMDIClient.ID_MDI_MENUWINDOW, LAYOUT_LEFT)
        FXMDIDeleteButton(menubar, self.mdiclient, FXMDIClient.ID_MDI_MENUCLOSE, FRAME_RAISED|LAYOUT_RIGHT)
        FXMDIRestoreButton(menubar, self.mdiclient, FXMDIClient.ID_MDI_MENURESTORE, FRAME_RAISED|LAYOUT_RIGHT)
        FXMDIMinimizeButton(menubar, self.mdiclient, FXMDIClient.ID_MDI_MENUMINIMIZE, FRAME_RAISED|LAYOUT_RIGHT)

        # Icon for MDI child windows
        self.mdiicon = self.iconResource('gifs/winapp.gif')

        # Make MDI window menu
        self.mdimenu = FXMDIMenu(self, self.mdiclient)

        # Make an MDI child window
        mdichild = FXMDIChild(self.mdiclient, "FOX GL Viewer", self.mdiicon, self.mdimenu, MDI_NORMAL, 30, 30, 300, 200)

        # Visual
        self.glvisual = FXGLVisual(self.getApp(), VISUAL_DOUBLEBUFFER)

        # Viewer
        viewer = FXGLViewer(mdichild, self.glvisual, None, 0,
            LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT)

        # Tab book with several panels
        panels = TabBook(frame, self)

        # Icons we'll soon need
        newdoc = self.iconResource('gifs/filenew.gif')
        opendoc = self.iconResource('gifs/fileopen.gif')
        savedoc = self.iconResource('gifs/filesave.gif')
        saveasdoc = self.iconResource('gifs/filesaveas.gif',0,IMAGE_ALPHAGUESS)

        # File menu
        filemenu = FXMenuPane(self)
        FXMenuTitle(menubar,"&File",None,filemenu)
        FXMenuCommand(filemenu,"&New...\tCtl-N\tCreate new document.",newdoc)
        FXMenuCommand(filemenu,"&Open...\tCtl-O\tOpen document file.",opendoc,self,self.ID_OPEN)
        FXMenuCommand(filemenu,"&Save\tCtl-S\tSave document.",savedoc)
        FXMenuCommand(filemenu,"Save &As...\t\tSave document to another file.",saveasdoc)
        FXMenuCommand(filemenu,"&Print...\t\tPrint document.")
        FXMenuCommand(filemenu,"&Dump...\t\tDump widgets.",None,self.getApp(),FXApp.ID_DUMP)
        FXMenuCommand(filemenu,"&Quit\tCtl-Q\tQuit the application.",None,self.getApp(),FXApp.ID_QUIT,MENU_DEFAULT)

        # Edit Menu
        editmenu = FXMenuPane(self)
        FXMenuTitle(menubar,"&Edit",None,editmenu)
        FXMenuCommand(editmenu,"Undo")
        FXMenuCommand(editmenu,"Copy",None,self.mdiclient,FXGLViewer.ID_COPY_SEL,MENU_AUTOGRAY)
        FXMenuCommand(editmenu,"Cut",None,self.mdiclient,FXGLViewer.ID_CUT_SEL,MENU_AUTOGRAY)
        FXMenuCommand(editmenu,"Paste",None,self.mdiclient,FXGLViewer.ID_PASTE_SEL,MENU_AUTOGRAY)
        FXMenuCommand(editmenu,"Delete",None,self.mdiclient,FXGLViewer.ID_DELETE_SEL,MENU_AUTOGRAY)

        # File manipulation
        FXButton(toolbar,"\tNew\tCreate new document.",newdoc,None,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar,"\tOpen\tOpen document file.",opendoc,self,self.ID_OPEN,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar,"\tSave\tSave document.",savedoc,None,0,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar,"\tSave As\tSave document to another file.",saveasdoc,self,self.ID_OPEN,FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tNew Folder\tNo comment",
            self.iconResource('gifs/newfolder.gif'),
            opts = FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Print
        FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,4,20)
        FXButton(toolbar, "\tPrint\tPrint document.",
            self.iconResource('gifs/printicon.gif'),
            opts = FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Editing
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,4,20)
        FXButton(toolbar, "\tCut",
            self.iconResource('gifs/cut.gif'),
            self.mdiclient, FXGLViewer.ID_CUT_SEL,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tCopy",
            self.iconResource('gifs/copy.gif'),
            self.mdiclient, FXGLViewer.ID_COPY_SEL,
            opts=BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tPaste",
            self.iconResource('gifs/paste.gif'),
            self.mdiclient, FXGLViewer.ID_PASTE_SEL,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Projections
        FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20)
        FXButton(toolbar, "\tPerspective\tSwitch to perspective projection.",
            self.iconResource('gifs/perspective.gif'),
            self.mdiclient, FXGLViewer.ID_PERSPECTIVE,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tParallel\tSwitch to parallel projection.",
            self.iconResource('gifs/parallel.gif'),
            self.mdiclient, FXGLViewer.ID_PARALLEL,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Shading model
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20)
        FXButton(toolbar, "\tNo shading\tTurn light sources off.",
            self.iconResource('gifs/nolight.gif'),
            self.mdiclient, FXGLCube.ID_SHADEOFF,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tFlat shading\tTurn on faceted (flat) shading.",
            self.iconResource('gifs/light.gif'),
            self.mdiclient, FXGLCube.ID_SHADEON,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tSmooth shading\tTurn on smooth shading.",
            self.iconResource('gifs/smoothlight.gif'),
            self.mdiclient, FXGLCube.ID_SHADESMOOTH,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20)
        FXToggleButton(toolbar, "\tToggle Light\tToggle light source.", None,
            self.iconResource('gifs/nolight.gif'),
            self.iconResource('gifs/light.gif'),
            opts = FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # View orientation
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20)
        FXButton(toolbar, "\tFront View\tView objects from the front.",
            self.iconResource('gifs/frontview.gif'),
            self.mdiclient, FXGLViewer.ID_FRONT,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tBack View\tView objects from behind.",
            self.iconResource('gifs/backview.gif'),
            self.mdiclient, FXGLViewer.ID_BACK,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tLeft View\tView objects from the left.",
            self.iconResource('gifs/leftview.gif'),
            self.mdiclient, FXGLViewer.ID_LEFT,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tRight View\tView objects from the right.",
            self.iconResource('gifs/rightview.gif'),
            self.mdiclient, FXGLViewer.ID_RIGHT,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tTop View\tView objects from the top.",
            self.iconResource('gifs/topview.gif', 0, IMAGE_OPAQUE),
            self.mdiclient, FXGLViewer.ID_TOP,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tBottom View\tView objects from below.",
            self.iconResource('gifs/bottomview.gif', 0, IMAGE_OPAQUE),
            self.mdiclient, FXGLViewer.ID_BOTTOM,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Miscellaneous buttons
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,8,20)
        FXButton(toolbar, None,
            self.iconResource('gifs/zoom.gif'),
            opts = FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "\tColors\tDisplay color dialog.",
            self.iconResource('gifs/colorpal.gif'),
            colordlg, FXWindow.ID_SHOW,
            FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, None,
            self.iconResource('gifs/camera.gif', 0, IMAGE_OPAQUE),
            opts = FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, None,
            self.iconResource('gifs/foxicon.gif'),
            opts = FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Dangerous delete a bit on the side
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20)
        FXButton(toolbar, "\tDelete\tDelete the selected object.",
            self.iconResource('gifs/kill.gif'),
            self.mdiclient, FXGLViewer.ID_DELETE_SEL,
            BUTTON_AUTOGRAY|FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # View menu
        viewmenu = FXMenuPane(self)
        FXMenuTitle(menubar,"&View",None,viewmenu)
        FXMenuCommand(viewmenu, "Parallel\t\tSwitch to parallel projection.",
            None, self.mdiclient, FXGLViewer.ID_PARALLEL, MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,
            "Perspective\t\tSwitch to perspective projection.", None,
            self.mdiclient,FXGLViewer.ID_PERSPECTIVE,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu, "&Front\t\tFront view.", None,
            self.mdiclient, FXGLViewer.ID_FRONT, MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"&Back\t\tBack view.",None,self.mdiclient,FXGLViewer.ID_BACK,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"&Left\t\tLeft view.",None,self.mdiclient,FXGLViewer.ID_LEFT,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"&Right\t\tRight view.",None,self.mdiclient,FXGLViewer.ID_RIGHT,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"&Top\t\tTop view.",None,self.mdiclient,FXGLViewer.ID_TOP,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"&Bottom\t\tBottom view.",None,self.mdiclient,FXGLViewer.ID_BOTTOM,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"F&it\t\tFit to view.",None,self.mdiclient,FXGLViewer.ID_FITVIEW,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"R&eset\t\tReset all viewing parameters",None,self.mdiclient,FXGLViewer.ID_RESETVIEW,MENU_AUTOGRAY)
        FXMenuCommand(viewmenu,"Lock\t\tLock view orientation",None,self.mdiclient,FXGLViewer.ID_LOCK,MENU_AUTOGRAY)

        # Window menu
        windowmenu = FXMenuPane(self)
        FXMenuTitle(menubar,"&Windows",None,windowmenu)
        FXMenuCommand(windowmenu,"New Viewer\t\tCreate new viewer window.",None,self,self.ID_NEWVIEWER)
        FXMenuCommand(windowmenu,"Tile Horizontally\t\tTile windows horizontally.",None,self.mdiclient,FXWindow.ID_MDI_TILEHORIZONTAL)
        FXMenuCommand(windowmenu,"Tile Vertically\t\tTile windows vertically.",None,self.mdiclient,FXWindow.ID_MDI_TILEVERTICAL)
        FXMenuCommand(windowmenu,"Cascade\t\tCascade windows.",None,self.mdiclient,FXWindow.ID_MDI_CASCADE)
        FXMenuCommand(windowmenu,"Toolbar",None,toolbar,FXWindow.ID_TOGGLESHOWN)
        FXMenuCommand(windowmenu,"Control panel",None,panels,FXWindow.ID_TOGGLESHOWN)
        FXMenuCommand(windowmenu,"Delete\t\tDelete current viewer window.",None,self.mdiclient,FXMDIClient.ID_MDI_CLOSE)

        # Help menu
        helpmenu = FXMenuPane(self)
        FXMenuTitle(menubar,"&Help",None,helpmenu,LAYOUT_RIGHT)
        FXMenuCommand(helpmenu,"&About FOX...\t\tDisplay FOX about panel.",None,self,self.ID_ABOUT,0)

        # Make a tool tip
        FXTooltip(self.getApp(), 0)

        # Status bar shows the mode
        statusbar.getStatusline().setTarget(self)
        statusbar.getStatusline().setSelector(self.ID_QUERY_MODE)

        # Make a scene!
        self.scene = self.makeScene()

        # Add scene to GL viewer
        viewer.setScene(self.scene)

    # Construct a scene object
    def makeScene(self):
        scene = FXGLGroup()
        scene.append(FXGLPoint(2,0,0))
        scene.append(FXGLPoint(0,2,0))
        scene.append(FXGLPoint(2,2,0))
        scene.append(FXGLPoint(0,0,0))
        scene.append(FXGLLine(0,0,0, 1,0,0))
        scene.append(FXGLLine(0,0,0, 0,1,0))
        scene.append(FXGLLine(0,0,0, 0,0,1))
        scene.append(FXGLLine(0,0,0, 1,1,1))
        gp2 = FXGLGroup()
        scene.append(gp2)
        gp2.append(FXGLCube(-1.0, 0.0, 0.0, 0.5, 0.5, 0.5))
        gp2.append(FXGLCube( 1.0, 0.0, 0.0, 0.5, 0.5, 0.5))
        gp2.append(FXGLCube( 0.0,-1.0, 0.0, 0.5, 0.5, 0.5))
        gp2.append(FXGLCube( 0.0, 1.0, 0.0, 0.5, 0.5, 1.5))
        gp2.append(FXGLCone( 1.0,-1.5, 0.0, 1.0, 0.5))
        gp2.append(FXGLCylinder(-1.0, 0.5, 0.0, 1.0, 0.5))
        gp2.append(FXGLSphere(1.0, 1.0, 0.0, 0.5))
        return scene

    # Create and initialize
    def create(self):
        FXMainWindow.create(self)
        self.show()

    # About
    def onCmdAbout(self,sender,sel,ptr):
        showModalInformationBox(self,MBOX_OK,"About FOX","FOX OpenGL Example.\nCopyright (C) 1998 Jeroen van der Zijp")
        return 1

    # Open
    def onCmdOpen(self,sender,sel,ptr):
        patterns = [
            "All Files",          "*",
            "C++ Source Files",   "*.[Cc][Pp][Pp]",
            "C++ Header Files",   "*.[Hh]",
            "Object Files",       "*.o",
            "HTML Header Files",  "*.[Hh][Tt][Mm][Ll]" ]
        open = FXFileDialog(self, "Open some file")
        open.setPatternList(patterns)
        if open.execute():
            pass
        return 1

    # Make new viewer
    def onCmdNewViewer(self,sender,sel,ptr):
        # Make a new child window
        mdichild = FXMDIChild(self.mdiclient,"FOX GL Viewer",self.mdiicon,self.mdimenu,MDI_NORMAL,30,30,300,200)

        # Create a new viewer
        viewer = FXGLViewer(mdichild,self.glvisual)

        # Change of scenery
        viewer.setScene(self.scene)

        # Create the child
        mdichild.create()
        return 1

    # Update status line text
    def onUpdMode(self, sender, sel, ptr):
        sender.setText("Ready.")
        return 1

def runme():
    import sys
    app = FXApp("GLViewer", "Test")
    app.init(sys.argv)
    GLViewWindow(app)
    app.create()
    app.run()

# Main program starts here
if __name__ == '__main__':
    runme()
