#! /usr/bin/env python

from FXPy import *
import os

# File filter pattens for the file dialog
patterns = [ "All Files", "*",
    "C++ Source Files", "*.cpp",
    "C Source Files", "*.c",
    "C++ Header Files", "*.h",
    "Text Files", "*.txt",
    "HTML Files", "*.html"]

# Main application window
class TextWindow(FXMainWindow):
    # Message IDs
    (ID_ABOUT,
    ID_OPEN,
    ID_REVERT,
    ID_SAVE,
    ID_SAVEAS,
    ID_NEW,
    ID_TITLE,
    ID_FONT,
    ID_QUIT,
    ID_PRINT,
    ID_TREELIST,
    ID_BACKCOLOR,
    ID_TEXTCOLOR,
    ID_SELBACKCOLOR,
    ID_SELTEXTCOLOR,
    ID_TOGGLE_WRAP,
    ID_SAVE_SETTINGS,
    ID_TEXT,
    ID_SEARCH,
    ID_REPLACE) = range(FXMainWindow.ID_LAST, FXMainWindow.ID_LAST+20)

    def loadIcon(self, filename):
        filename = os.path.join('gifs', filename)
        return FXGIFIcon(self.getApp(), open(filename, 'rb').read())

    # Constructor
    def __init__(self, app):
        FXMainWindow.__init__(self, app, "FOX Text Editor: - untitled",
            w=850, h=600, hs=0, vs=0)

        # Set up message map
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_ABOUT,               TextWindow.onCmdAbout)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_REVERT,              TextWindow.onCmdRevert)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_REVERT,              TextWindow.onUpdRevert)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_OPEN,                TextWindow.onCmdOpen)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_SAVE,                TextWindow.onCmdSave)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_SAVE,                TextWindow.onUpdSave)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_SAVEAS,              TextWindow.onCmdSaveAs)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_NEW,                 TextWindow.onCmdNew)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_TITLE,               TextWindow.onUpdTitle)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_FONT,                TextWindow.onCmdFont)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_QUIT,                TextWindow.onCmdQuit)
        FXMAPFUNC(self, SEL_SIGNAL,  TextWindow.ID_QUIT,                TextWindow.onCmdQuit)
        FXMAPFUNC(self, SEL_CLOSE,   TextWindow.ID_TITLE,               TextWindow.onCmdQuit)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_PRINT,               TextWindow.onCmdPrint)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_TREELIST,            TextWindow.onCmdTreeList)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_BACKCOLOR,           TextWindow.onCmdBackColor)
        FXMAPFUNC(self, SEL_CHANGED, TextWindow.ID_BACKCOLOR,           TextWindow.onCmdBackColor)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_BACKCOLOR,           TextWindow.onUpdBackColor)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_SELBACKCOLOR,        TextWindow.onCmdSelBackColor)
        FXMAPFUNC(self, SEL_CHANGED, TextWindow.ID_SELBACKCOLOR,        TextWindow.onCmdSelBackColor)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_SELBACKCOLOR,        TextWindow.onUpdSelBackColor)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_TEXTCOLOR,           TextWindow.onCmdTextColor)
        FXMAPFUNC(self, SEL_CHANGED, TextWindow.ID_TEXTCOLOR,           TextWindow.onCmdTextColor)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_TEXTCOLOR,           TextWindow.onUpdTextColor)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_SELTEXTCOLOR,        TextWindow.onCmdSelTextColor)
        FXMAPFUNC(self, SEL_CHANGED, TextWindow.ID_SELTEXTCOLOR,        TextWindow.onCmdSelTextColor)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_SELTEXTCOLOR,        TextWindow.onUpdSelTextColor)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_TOGGLE_WRAP,         TextWindow.onCmdWrap)
        FXMAPFUNC(self, SEL_UPDATE,  TextWindow.ID_TOGGLE_WRAP,         TextWindow.onUpdWrap)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_SAVE_SETTINGS,       TextWindow.onCmdSaveSettings)
        FXMAPFUNC(self, SEL_INSERTED,TextWindow.ID_TEXT,                TextWindow.onTextInserted)
        FXMAPFUNC(self, SEL_DELETED, TextWindow.ID_TEXT,                TextWindow.onTextDeleted)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_SEARCH,              TextWindow.onCmdSearch)
        FXMAPFUNC(self, SEL_COMMAND, TextWindow.ID_REPLACE,             TextWindow.onCmdReplace)

        # Set main window icons
        bigp = self.loadIcon('bigpenguin.gif')
        tinyp = self.loadIcon('penguin.gif')
        self.setIcon(bigp)
        self.setMiniIcon(tinyp)

        # Make main window set myself as the target
        self.setTarget(self)
        self.setSelector(self.ID_TITLE)

        # Help window
        # self.helpwindow = HelpWindow(self)
        self.helpwindow = None

        # Make color dialog
        self.colordlg = FXColorDialog(self, "Color Dialog")

        # Make menu bar
        menubar = FXMenubar(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)

        # Tool bar
        dragshell = FXToolbarShell(self, FRAME_RAISED|FRAME_THICK, 100, 100)
        toolbar = FXToolbar(self, dragshell,
            LAYOUT_SIDE_TOP|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT,
            pl=4, pr=4, pt=2, pb=2)

        # Status bar
        statusbar = FXStatusbar(self, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER)

        # Info about the editor
        FXButton(statusbar, "\tThe FOX Text Editor\tAbout the FOX Text Editor", self.loadIcon('small_gif.gif'),
                 self, TextWindow.ID_ABOUT, LAYOUT_FILL_Y|LAYOUT_RIGHT)

        # File menu
        filemenu = FXMenuPane(menubar)
        FXMenuTitle(menubar, "&File", None, filemenu)

        # Edit Menu
        editmenu = FXMenuPane(menubar)
        FXMenuTitle(menubar, "&Edit", None, editmenu)

        # Options Menu
        optionsmenu = FXMenuPane(menubar)
        FXMenuTitle(menubar, "&Options", None, optionsmenu)

        # View menu
        viewmenu = FXMenuPane(menubar)
        FXMenuTitle(menubar, "&View", None, viewmenu)

        # Help menu
        helpmenu = FXMenuPane(menubar)
        FXMenuTitle(menubar, "&Help", None, helpmenu, LAYOUT_RIGHT)

        # Splitter
        splitter = FXSplitter(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING)

        # Sunken border for tree
        self.treebox = FXVerticalFrame(splitter, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)

        # Make the tree
        treeframe = FXHorizontalFrame(self.treebox, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                      0, 0, 0, 0, 0, 0, 0, 0)
        self.dirlist = FXDirList(treeframe, 0, self, self.ID_TREELIST, DIRLIST_SHOWFILES|TREELIST_BROWSESELECT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        filterframe = FXHorizontalFrame(self.treebox, LAYOUT_FILL_X)
        FXLabel(filterframe, "Filter:")
        filter = FXTextField(filterframe, 10, self.dirlist, FXDirList.ID_SET_PATTERN,
                             JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X)

        # Sunken frame for text area
        textbox = FXHorizontalFrame(splitter, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                    0, 0, 0, 0, 0, 0, 0, 0)

        # Make editor window
        self.editor = FXText(textbox, self, self.ID_TEXT, opts = LAYOUT_FILL_X|LAYOUT_FILL_Y)

        # Show column number in status bar
        FXTextField(statusbar, 4, self.editor, FXText.ID_CURSOR_COLUMN,
                    FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,
                    0, 0, 0, 0, 1, 1, 0, 0)

        # Caption before number
        FXLabel(statusbar, "Col:", None, LAYOUT_RIGHT|LAYOUT_CENTER_Y)

        # Show line number in status bar
        FXTextField(statusbar, 4, self.editor, FXText.ID_CURSOR_ROW,
                    FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,
                    0, 0, 0, 0, 1, 1, 0, 0)

        # Caption before number
        FXLabel(statusbar, "Line:", None, LAYOUT_RIGHT|LAYOUT_CENTER_Y)

        # Text color
        self.textwell = FXColorWell(statusbar, FXRGB(0,0,0), self, self.ID_TEXTCOLOR,
                                    FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
        self.backwell = FXColorWell(statusbar, FXRGB(0,0,0), self, self.ID_BACKCOLOR,
                                    FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
        self.seltextwell = FXColorWell(statusbar, FXRGB(0,0,0), self, self.ID_SELTEXTCOLOR,
                                       FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
        self.selbackwell = FXColorWell(statusbar, FXRGB(0,0,0), self, self.ID_SELBACKCOLOR,
                                       FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)

        self.textwell.setTipText("Text")
        self.backwell.setTipText("Background")
        self.seltextwell.setTipText("Selection")
        self.selbackwell.setTipText("Selection Background")

        self.textwell.setHelpText("Change text color.")
        self.backwell.setHelpText("Change background color.")
        self.seltextwell.setHelpText("Change selection color.")
        self.selbackwell.setHelpText("Change selection background color.")

        # Toolbar buttons: File manipulation
        FXButton(toolbar, "&New\tNew\tCreate new document.",
            self.loadIcon('filenew.gif'), self, self.ID_NEW,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "&Open\tOpen\tOpen document file.",
            self.loadIcon('fileopen.gif'), self, self.ID_OPEN,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "&Save\tSave\tSave document.",
            self.loadIcon('filesave.gif'), self, self.ID_SAVE,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "S&ave As\tSave As\tSave document to another file.",
            self.loadIcon('filesaveas.gif'), self, self.ID_SAVEAS,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Toolbar buttons: Print
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 5, 20)
        FXButton(toolbar, "&Print\tPrint\tPrint document.",
            self.loadIcon('printicon.gif'), self, self.ID_PRINT,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Toolbar buttons: Editing
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 5, 20)
        FXButton(toolbar, "Cut\tCut", self.loadIcon('cut.gif'),
            self.editor, FXText.ID_CUT_SEL,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "Copy\tCopy", self.loadIcon('copy.gif'),
            self.editor, FXText.ID_COPY_SEL,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "Paste\tPaste", self.loadIcon('paste.gif'),
            self.editor, FXText.ID_PASTE_SEL,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # Color
        FXFrame(toolbar,
            LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 5, 20)
        FXButton(toolbar, "&Colors\tColors\tDisplay color dialog.",
            self.loadIcon('colorpal.gif'), self.colordlg, FXWindow.ID_SHOW,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)
        FXButton(toolbar, "&Fonts\tFonts\tDisplay font dialog.",
            self.loadIcon('fonts.gif'), self, self.ID_FONT,
            ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT)

        # File Menu entries
        FXMenuCommand(filemenu, "&Open...\t\tOpen document file.", None, self, self.ID_OPEN)
        FXMenuCommand(filemenu, "&Revert...\t\tRevert to saved.", None, self, self.ID_REVERT)
        FXMenuCommand(filemenu, "&New...\t\tCreate new document.", None, self, self.ID_NEW)
        FXMenuCommand(filemenu, "&Save\t\tSave document.", None, self, self.ID_SAVE)
        FXMenuCommand(filemenu, "Save &As...\t\tSave document to another file.", None, self, self.ID_SAVEAS)
        FXMenuCommand(filemenu, "&Print...\t\tPrint document.", None, self, self.ID_PRINT)
        FXMenuCommand(filemenu, "&Editable\t\tDocument editable.", None, self.editor, FXText.ID_TOGGLE_EDITABLE)
        FXMenuCommand(filemenu, "&Iconify\t\tIconify editor.", None, self, FXTopWindow.ID_ICONIFY)
        FXMenuCommand(filemenu, "&Quit\tCtl-Q", None, self, self.ID_QUIT, MENU_DEFAULT)

        # Edit Menu entries
        FXMenuCommand(editmenu, "&Undo\tCtl-Z\tUndo last change.")
        FXMenuCommand(editmenu, "&Redo\tCtl-R\tRedo last undo.")
        FXMenuCommand(editmenu, "&Copy\tCtl-C\tCopy selection to clipboard.", None, self.editor, FXText.ID_COPY_SEL)
        FXMenuCommand(editmenu, "C&ut\tCtl-X\tCut selection to clipboard.", None, self.editor, FXText.ID_CUT_SEL)
        FXMenuCommand(editmenu, "&Paste\tCtl-V\tPaste from clipboard.", None, self.editor, FXText.ID_PASTE_SEL)
        FXMenuCommand(editmenu, "&Delete\t\tDelete selection.", None, self.editor, FXText.ID_DELETE_SEL)
        FXMenuSeparator(editmenu)
        FXMenuCommand(editmenu, "&Search...\tCtl-F\tSearch for a string.", None, self, TextWindow.ID_SEARCH)
        FXMenuCommand(editmenu, "R&eplace...\tCtl-R\tSearch for a string.", None, self, TextWindow.ID_REPLACE)

        # Options menu
        FXMenuCommand(optionsmenu, "Font...\t\tChange text font.", None, self, self.ID_FONT)
        FXMenuCommand(optionsmenu, "Colors...\t\tChange text colors.", None, self.colordlg, FXWindow.ID_SHOW)
        FXMenuCommand(optionsmenu, "Word Wrap\t\tToggle word wrap.", None, self, self.ID_TOGGLE_WRAP)
        FXMenuCommand(optionsmenu, "Overstrike\t\tToggle overstrike mode.", None, self.editor, FXText.ID_TOGGLE_OVERSTRIKE)
        FXMenuCommand(optionsmenu, "Save Settings...\t\tSave settings now.", None, self, self.ID_SAVE_SETTINGS)

        # View Menu entries
        FXMenuCommand(viewmenu, "File Browser\t\tDisplay file list.", None, self.treebox, FXWindow.ID_TOGGLESHOWN)
        FXMenuCommand(viewmenu, "Hidden Files\t\tShow hidden files and directories.", None, self.dirlist, FXDirList.ID_TOGGLE_HIDDEN)
        FXMenuCommand(viewmenu, "Toolbar\t\tDisplay toolbar.", None, toolbar, FXWindow.ID_TOGGLESHOWN)
        FXMenuCommand(viewmenu, "Status line\t\tDisplay status line.", None, statusbar, FXWindow.ID_TOGGLESHOWN)

        # Help Menu entries
        FXMenuCommand(helpmenu, "&Help...\t\tDisplay help information.", self.loadIcon('help_gif.gif'), self.helpwindow, self.ID_SHOW, 0);
        FXMenuSeparator(helpmenu)
        FXMenuCommand(helpmenu, "&About FOX...", None, self, self.ID_ABOUT, 0)

        # Make a tool tip
        FXTooltip(self.getApp(), 0)

        # Initialize file name
        self.filename = "untitled"
        self.namechanged = 0
        self.modified = 1

    # Load file
    def loadFile(self, file):
        if not os.path.exists(file):
            showModalErrorBox(self, MBOX_OK, "Error Loading File",
                "Non-existing file: " + file)
            return 0
        try:
            self.getApp().beginWaitCursor()
            text = open(file, 'r').read()
        except IOError:
            getApp().endWaitCursor()
            showModalErrorBox(self, MBOX_OK, "Error Loading File",
                "Unable to open file: " + file)
            return 0
        self.getApp().endWaitCursor()
        self.editor.setText(text)
        self.editor.setModified(0)
        return 1

    # Save file
    def saveFile(self, file):
        text = self.editor.getText()
        try:
            self.getApp().beginWaitCursor()
            open(file, 'w').write(text)
        except IOError:
            self.getApp().endWaitCursor()
            showModalErrorBox(self, MBOX_OK, "Error Saving File",
                              "Unable to open file: " + file)
            return 0
        self.getApp().endWaitCursor()
        self.editor.setModified(0)
        return 1

    # Read settings from registry
    def readRegistry(self):
        # Get colors
        textback = self.getApp().reg().readColorEntry("SETTINGS", "textbackground", FXRGB(255,255,255))
        textfore = self.getApp().reg().readColorEntry("SETTINGS", "textforeground", FXRGB(0,0,0))
        textselback = self.getApp().reg().readColorEntry("SETTINGS", "textselbackground", FXRGB(0,0,128))
        textselfore = self.getApp().reg().readColorEntry("SETTINGS", "textselforeground", FXRGB(255,255,255))

        # Get size
        xx = self.getApp().reg().readIntEntry("SETTINGS", "x", 5)
        yy = self.getApp().reg().readIntEntry("SETTINGS", "y", 5)
        ww = self.getApp().reg().readIntEntry("SETTINGS", "width", 600)
        hh = self.getApp().reg().readIntEntry("SETTINGS", "height", 400)

        # Hidden files shown?
        hiddenfiles = self.getApp().reg().readIntEntry("SETTINGS", "showhiddenfiles", 0)
        self.dirlist.showHiddenFiles(hiddenfiles)

        # Showing the tree?
        hidetree = self.getApp().reg().readIntEntry('SETTINGS', 'hidetree', 1)

        # Width of the tree
        treewidth = self.getApp().reg().readIntEntry('SETTINGS', 'treewidth', 100)

        # Word wrapping
        wrapping = self.getApp().reg().readIntEntry('SETTINGS', 'wordwrap', 0)

        # Change the colors
        self.editor.setTextColor(textfore)
        self.dirlist.setTextColor(textfore)
        self.editor.setBackColor(textback)
        self.dirlist.setBackColor(textback)
        self.editor.setSelBackColor(textselback)
        self.dirlist.setSelBackColor(textselback)
        self.editor.setSelTextColor(textselfore)
        self.dirlist.setSelTextColor(textselfore)

        # Hide the tree if asked for
        if hidetree:
            self.treebox.hide()

        # Set tree width
        self.treebox.setWidth(treewidth)

        # Open toward file
        self.dirlist.setCurrentFile(self.filename)

        # Wrap mode
        if wrapping:
            self.editor.setTextStyle(self.editor.getTextStyle()|TEXT_WORDWRAP)
        else:
            self.editor.setTextStyle(self.editor.getTextStyle()&~TEXT_WORDWRAP)

        # Reposition window
        self.position(xx, yy, ww, hh)

    # Save settings to registry
    def writeRegistry(self):
        # Write changed colors back to registry
        self.getApp().reg().writeColorEntry("SETTINGS", "textbackground", self.editor.getBackColor())
        self.getApp().reg().writeColorEntry("SETTINGS", "textforeground", self.editor.getTextColor())
        self.getApp().reg().writeColorEntry("SETTINGS", "textselbackground", self.editor.getSelBackColor())
        self.getApp().reg().writeColorEntry("SETTINGS", "textselforeground", self.editor.getSelTextColor())

        # Write new window size back to registry
        self.getApp().reg().writeIntEntry("SETTINGS", "x", self.getX())
        self.getApp().reg().writeIntEntry("SETTINGS", "y", self.getY())
        self.getApp().reg().writeIntEntry("SETTINGS", "width", self.getWidth())
        self.getApp().reg().writeIntEntry("SETTINGS", "height", self.getHeight())

        # Were showing hidden files?
        self.getApp().reg().writeIntEntry("SETTINGS", "showhidden", self.dirlist.showHiddenFiles())

        # Was tree shown?
        self.getApp().reg().writeIntEntry("SETTINGS", "hidetree", 1 - self.treebox.shown())

        # Width of tree
        self.getApp().reg().writeIntEntry("SETTINGS", "treewidth", self.treebox.getWidth())

        # Wrap mode
        wordwrap = (self.editor.getTextStyle() & TEXT_WORDWRAP) != 0
        self.getApp().reg().writeIntEntry("SETTINGS", "wordwrap", wordwrap)

    # About box
    def onCmdAbout(self, sender, sel, ptr):
        icon = self.loadIcon('bigpenguin.gif')
        about = FXMessageBox(self, "About Text Edit", "Text Editor Demonstrates the FOX Text Widget.\n\nUsing the FOX C++ GUI Library (http://www.cfdrc.com/FOX/fox.html)\n\nCopyright (C) 2000 Jeroen van der Zijp (jvz@cfdrc.com)", icon, MBOX_OK|DECOR_TITLE|DECOR_BORDER)
        about.execute()
        return 1

    # Change font
    def onCmdFont(self, sender, sel, ptr):
        fontdlg = FXFontDialog(self, "Change Font", DECOR_BORDER|DECOR_TITLE)
        oldfont = self.editor.getFont()
        fontdesc = oldfont.getFontDesc()
        fontdlg.setFontSelection(fontdesc)
        if fontdlg.execute():
            fontdesc = fontdlg.getFontSelection()
            newfont = FXFont(self.getApp(), fontdesc)
            newfont.create()
            self.editor.setFont(newfont)
        self.editor.setFocus()
        return 1

    # Toggle wrap mode
    def onCmdWrap(self, sender, sel, ptr):
        self.editor.setTextStyle(self.editor.getTextStyle()^TEXT_WORDWRAP)
        return 1

    # Save settings
    def onCmdSaveSettings(self, sender, sel, ptr):
        self.writeRegistry()
        self.getApp().reg().write()
        return 1

    # Update toggle wrap mode
    def onUpdWrap(self, sender, sel, ptr):
        if self.editor.getTextStyle()&TEXT_WORDWRAP:
            sender.handle(self, MKUINT(FXWindow.ID_CHECK, SEL_COMMAND), None)
        else:
            sender.handle(self, MKUINT(FXWindow.ID_UNCHECK, SEL_COMMAND), None)
        return 1

    # Revert to saved
    def onCmdRevert(self, sender, sel, ptr):
        self.loadFile(self.filename)
        return 1

    # Revert update
    def onUpdRevert(self, sender, sel, ptr):
        if self.modified != self.editor.isModified():
            self.modified = self.editor.isModified()
            if self.modified:
                msg = FXWindow.ID_ENABLE
            else:
                msg = FXWindow.ID_DISABLE
            sender.handle(self, MKUINT(msg, SEL_COMMAND), ptr)
        return 1

    # Open a file
    def onCmdOpen(self, sender, sel, ptr):
        if self.editor.isModified():
            answer = showModalQuestionBox(self,MBOX_YES_NO_CANCEL,
                "Unsaved Document", "Save current document to file?")
            if answer == MBOX_CLICKED_CANCEL:
                return 1
            if answer == MBOX_CLICKED_YES:
                if not self.namechanged:
                    savedialog = FXFileDialog(self, "Save Document")
                    if savedialog.execute():
                        if os.path.exists(savedialog.getFilename()):
                            if MBOX_CLICKED_NO == showModalQuestionBox(self,MBOX_YES_NO, "Overwrite Document", "Overwrite existing document?"):
                                return 1
                        self.filename = savedialog.getFilename()
                self.saveFile(self.filename)

        open = FXFileDialog(self, "Open Document")
        open.setFilename(self.filename)
        open.setPatternList(patterns)
        if open.execute():
            self.filename = open.getFilename();
            self.loadFile(self.filename)
            self.namechanged = 1
        return 1

    # Save
    def onCmdSave(self, sender, sel, ptr):
        if not self.namechanged:
            return self.onCmdSaveAs(sender, sel, ptr)
        self.saveFile(self.filename)
        return 1

    # Save Update
    def onUpdSave(self, sender, sel, ptr):
        if self.editor.isModified():
            msg = FXWindow.ID_ENABLE
        else:
            msg = FXWindow.ID_DISABLE
        sender.handle(self, MKUINT(msg, SEL_COMMAND), ptr)
        return 1

    # Save As
    def onCmdSaveAs(self, sender, sel, ptr):
        savedialog = FXFileDialog(self, "Save Document")
        if savedialog.execute():
            if os.path.exists(savedialog.getFilename()):
                if MBOX_CLICKED_NO == showModalQuestionBox(self,MBOX_YES_NO,"Overwrite Document","Overwrite existing document?."):
                    return 1
            self.filename = savedialog.getFilename()
            self.saveFile(self.filename)
            self.namechanged = 1
        return 1

    # New
    def onCmdNew(self, sender, sel, ptr):
        if self.editor.isModified():
            answer = showModalQuestionBox(self,MBOX_YES_NO_CANCEL,"Unsaved Document","Save current document to file?")
            if answer == MBOX_CLICKED_CANCEL:
                return 1
            if answer == MBOX_CLICKED_YES:
                if not self.namechanged:
                    savedialog = FXFileDialog(self, "Save Document")
                    if savedialog.execute():
                        if os.path.exists(savedialog.getFilename()):
                            if MBOX_CLICKED_NO == showModalQuestionBox(self,MBOX_YES_NO,"Overwrite Document","Overwrite existing document?."):
                                return 1
                        self.filename = savedialog.getFilename()
                self.saveFile(self.filename)
        self.filename = "untitled"
        self.editor.setText("")
        self.namechanged = 0
        self.editor.setModified(0)
        return 1

    # Quit
    def onCmdQuit(self, sender, sel, ptr):
        if self.editor.isModified():
            answer = showModalQuestionBox(self,MBOX_YES_NO_CANCEL,"Unsaved Document","Save current document to file?")
            if answer == MBOX_CLICKED_CANCEL:
                return 1
            if answer == MBOX_CLICKED_YES:
                if not self.namechanged:
                    savedialog = FXFileDialog(self, "Save Document")
                    if savedialog.execute():
                        if os.path.exists(savedialog.getFilename()):
                            if MBOX_CLICKED_NO == showModalQuestionBox(self,MBOX_YES_NO,"Overwrite Document","Overwrite existing document?."):
                                return 1
                        self.filename = savedialog.getFilename()
                self.saveFile(self.filename)
        self.writeRegistry()
        self.getApp().exit(0)
        return 1

    # Update title
    def onUpdTitle(self, sender, sel, ptr):
        title = "FOX Text Editor:- " + self.filename
        if self.editor.isModified():
            title = title + "*"
        sender.setTitle(title)
        return 1

    # Print the text
    def onCmdPrint(self, sender, sel, ptr):
        dlg = FXPrintDialog(self, "Print File")
        if dlg.execute():
            printer = dlg.getPrinter()
        return 1

    # File selected from the tree list
    def onCmdTreeList(self, sender, sel, item):
        if (not item) or (not self.dirlist.isItemFile(item)):
            return 1
        if self.editor.isModified():
            answer = showModalQuestionBox(self,MBOX_YES_NO_CANCEL,
                "Unsaved Document", "Save current document to file?")
            if answer == MBOX_CLICKED_CANCEL: return 1
            if answer == MBOX_CLICKED_YES:
                if not namechanged:
                    savedialog = FXFileDialog(self, "Save Document")
                    if not savedialog.execute(): return 1
                    if os.path.exists(savedialog.getFilename()):
                        if MBOX_CLICKED_NO == showModalQuestionBox(self,MBOX_YES_NO, "Overwrite Document", "Overwrite existing document?"): return 1
                    self.filename = savedialog.getFilename()
                self.savefile(self.filename)
        self.filename = self.dirlist.getItemPathname(item)
        self.loadFile(self.filename)
        self.namechanged = 1
        return 1

    # Change both text and tree background color
    def onCmdBackColor(self, sender, sel, ptr):
        color = sender.getRGBA()
        self.editor.setBackColor(color)
        self.dirlist.setBackColor(color)
        return 1

    # Update background color
    def onUpdBackColor(self, sender, sel, ptr):
        self.editor.handle(sender, MKUINT(FXText.ID_BACK_COLOR, SEL_UPDATE), ptr)
        return 1

    # Change both text and tree selected background color
    def onCmdSelBackColor(self, sender, sel, ptr):
        color = sender.getRGBA()
        self.editor.setSelBackColor(color)
        self.dirlist.setSelBackColor(color)
        return 1

    # Update selected background color
    def onUpdSelBackColor(self, sender, sel, ptr):
        self.editor.handle(sender, MKUINT(FXText.ID_SELBACK_COLOR, SEL_UPDATE), ptr)
        return 1

    # Change both text and tree text color
    def onCmdTextColor(self, sender, sel, ptr):
        color = sender.getRGBA()
        self.editor.setTextColor(color)
        self.dirlist.setTextColor(color)
        return 1

    # Update text color
    def onUpdTextColor(self, sender, sel, ptr):
        self.editor.handle(sender, MKUINT(FXText.ID_TEXT_COLOR, SEL_UPDATE), ptr)
        return 1

    # Change both text and tree selected text color
    def onCmdSelTextColor(self, sender, sel, ptr):
        color = sender.getRGBA()
        self.editor.setSelTextColor(color)
        self.dirlist.setSelTextColor(color)
        return 1

    # Update selected text color
    def onUpdSelTextColor(self, sender, sel, ptr):
        self.editor.handle(sender, MKUINT(FXText.ID_SELTEXT_COLOR, SEL_UPDATE), ptr)
        return 1

    # Text was inserted
    def onTextInserted(self, sender, sel, ptr):
        return 1

    # Text was deleted
    def onTextDeleted(self, sender, sel, ptr):
        return 1

    # Search text
    def onCmdSearch(self, sender, sel, ptr):
        search = FXSearchDialog(self, "Search Text")
        search.setSearchDir(1)
        search.setSearchText('Jeroen')
        search.setSearchMode(SEARCHMODE_IGNORECASE)
        search.execute()
        return 1

    # Replace text
    def onCmdReplace(self, sender, sel, ptr):
        replace = FXReplaceDialog(self, "Replace Text")
        replace.execute()
        return 1

    # Start the ball rolling
    def start(self, argv):
        if len(argv) > 1:
            self.filename = argv[1]
            self.loadfile(self.filename)
            self.namechanged = 1

    # Create and show window
    def create(self):
        self.readRegistry()
        FXMainWindow.create(self)
        self.show()

def runme():
    import sys
    app = FXApp("TextEdit", "Test")
    app.init(sys.argv)
    window = TextWindow(app)
    app.create()
    window.start(sys.argv)
    app.run()

# Main program starts here
if __name__ == '__main__':
    runme()
